# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..types import (
    ServicePlan,
    TrafficType,
    UsagePaymentMethod,
    outbound_voice_profile_list_params,
    outbound_voice_profile_create_params,
    outbound_voice_profile_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.service_plan import ServicePlan
from ..types.traffic_type import TrafficType
from ..types.usage_payment_method import UsagePaymentMethod
from ..types.outbound_voice_profile import OutboundVoiceProfile
from ..types.outbound_call_recording_param import OutboundCallRecordingParam
from ..types.outbound_voice_profile_create_response import OutboundVoiceProfileCreateResponse
from ..types.outbound_voice_profile_delete_response import OutboundVoiceProfileDeleteResponse
from ..types.outbound_voice_profile_update_response import OutboundVoiceProfileUpdateResponse
from ..types.outbound_voice_profile_retrieve_response import OutboundVoiceProfileRetrieveResponse

__all__ = ["OutboundVoiceProfilesResource", "AsyncOutboundVoiceProfilesResource"]


class OutboundVoiceProfilesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OutboundVoiceProfilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return OutboundVoiceProfilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OutboundVoiceProfilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return OutboundVoiceProfilesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        billing_group_id: Optional[str] | Omit = omit,
        call_recording: OutboundCallRecordingParam | Omit = omit,
        calling_window: outbound_voice_profile_create_params.CallingWindow | Omit = omit,
        concurrent_call_limit: Optional[int] | Omit = omit,
        daily_spend_limit: str | Omit = omit,
        daily_spend_limit_enabled: bool | Omit = omit,
        enabled: bool | Omit = omit,
        max_destination_rate: float | Omit = omit,
        service_plan: ServicePlan | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        traffic_type: TrafficType | Omit = omit,
        usage_payment_method: UsagePaymentMethod | Omit = omit,
        whitelisted_destinations: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutboundVoiceProfileCreateResponse:
        """
        Create an outbound voice profile.

        Args:
          name: A user-supplied name to help with organization.

          billing_group_id: The ID of the billing group associated with the outbound proflile. Defaults to
              null (for no group assigned).

          calling_window: (BETA) Specifies the time window and call limits for calls made using this
              outbound voice profile. Note that all times are UTC in 24-hour clock time.

          concurrent_call_limit: Must be no more than your global concurrent call limit. Null means no limit.

          daily_spend_limit: The maximum amount of usage charges, in USD, you want Telnyx to allow on this
              outbound voice profile in a day before disallowing new calls.

          daily_spend_limit_enabled: Specifies whether to enforce the daily_spend_limit on this outbound voice
              profile.

          enabled: Specifies whether the outbound voice profile can be used. Disabled profiles will
              result in outbound calls being blocked for the associated Connections.

          max_destination_rate: Maximum rate (price per minute) for a Destination to be allowed when making
              outbound calls.

          service_plan: Indicates the coverage of the termination regions.

          traffic_type: Specifies the type of traffic allowed in this profile.

          usage_payment_method: Setting for how costs for outbound profile are calculated.

          whitelisted_destinations: The list of destinations you want to be able to call using this outbound voice
              profile formatted in alpha2.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/outbound_voice_profiles",
            body=maybe_transform(
                {
                    "name": name,
                    "billing_group_id": billing_group_id,
                    "call_recording": call_recording,
                    "calling_window": calling_window,
                    "concurrent_call_limit": concurrent_call_limit,
                    "daily_spend_limit": daily_spend_limit,
                    "daily_spend_limit_enabled": daily_spend_limit_enabled,
                    "enabled": enabled,
                    "max_destination_rate": max_destination_rate,
                    "service_plan": service_plan,
                    "tags": tags,
                    "traffic_type": traffic_type,
                    "usage_payment_method": usage_payment_method,
                    "whitelisted_destinations": whitelisted_destinations,
                },
                outbound_voice_profile_create_params.OutboundVoiceProfileCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OutboundVoiceProfileCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutboundVoiceProfileRetrieveResponse:
        """
        Retrieves the details of an existing outbound voice profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/outbound_voice_profiles/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OutboundVoiceProfileRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        name: str,
        billing_group_id: Optional[str] | Omit = omit,
        call_recording: OutboundCallRecordingParam | Omit = omit,
        calling_window: outbound_voice_profile_update_params.CallingWindow | Omit = omit,
        concurrent_call_limit: Optional[int] | Omit = omit,
        daily_spend_limit: str | Omit = omit,
        daily_spend_limit_enabled: bool | Omit = omit,
        enabled: bool | Omit = omit,
        max_destination_rate: float | Omit = omit,
        service_plan: ServicePlan | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        traffic_type: TrafficType | Omit = omit,
        usage_payment_method: UsagePaymentMethod | Omit = omit,
        whitelisted_destinations: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutboundVoiceProfileUpdateResponse:
        """
        Updates an existing outbound voice profile.

        Args:
          name: A user-supplied name to help with organization.

          billing_group_id: The ID of the billing group associated with the outbound proflile. Defaults to
              null (for no group assigned).

          calling_window: (BETA) Specifies the time window and call limits for calls made using this
              outbound voice profile.

          concurrent_call_limit: Must be no more than your global concurrent call limit. Null means no limit.

          daily_spend_limit: The maximum amount of usage charges, in USD, you want Telnyx to allow on this
              outbound voice profile in a day before disallowing new calls.

          daily_spend_limit_enabled: Specifies whether to enforce the daily_spend_limit on this outbound voice
              profile.

          enabled: Specifies whether the outbound voice profile can be used. Disabled profiles will
              result in outbound calls being blocked for the associated Connections.

          max_destination_rate: Maximum rate (price per minute) for a Destination to be allowed when making
              outbound calls.

          service_plan: Indicates the coverage of the termination regions.

          traffic_type: Specifies the type of traffic allowed in this profile.

          usage_payment_method: Setting for how costs for outbound profile are calculated.

          whitelisted_destinations: The list of destinations you want to be able to call using this outbound voice
              profile formatted in alpha2.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/outbound_voice_profiles/{id}",
            body=maybe_transform(
                {
                    "name": name,
                    "billing_group_id": billing_group_id,
                    "call_recording": call_recording,
                    "calling_window": calling_window,
                    "concurrent_call_limit": concurrent_call_limit,
                    "daily_spend_limit": daily_spend_limit,
                    "daily_spend_limit_enabled": daily_spend_limit_enabled,
                    "enabled": enabled,
                    "max_destination_rate": max_destination_rate,
                    "service_plan": service_plan,
                    "tags": tags,
                    "traffic_type": traffic_type,
                    "usage_payment_method": usage_payment_method,
                    "whitelisted_destinations": whitelisted_destinations,
                },
                outbound_voice_profile_update_params.OutboundVoiceProfileUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OutboundVoiceProfileUpdateResponse,
        )

    def list(
        self,
        *,
        filter: outbound_voice_profile_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal[
            "enabled",
            "-enabled",
            "created_at",
            "-created_at",
            "name",
            "-name",
            "service_plan",
            "-service_plan",
            "traffic_type",
            "-traffic_type",
            "usage_payment_method",
            "-usage_payment_method",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[OutboundVoiceProfile]:
        """
        Get all outbound voice profiles belonging to the user that match the given
        filters.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[name][contains]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code>-</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>name</code>: sorts the result by the
                  <code>name</code> field in ascending order.
                </li>

                <li>
                  <code>-name</code>: sorts the result by the
                  <code>name</code> field in descending order.
                </li>
              </ul> <br/>

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/outbound_voice_profiles",
            page=SyncDefaultFlatPagination[OutboundVoiceProfile],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    outbound_voice_profile_list_params.OutboundVoiceProfileListParams,
                ),
            ),
            model=OutboundVoiceProfile,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutboundVoiceProfileDeleteResponse:
        """
        Deletes an existing outbound voice profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/outbound_voice_profiles/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OutboundVoiceProfileDeleteResponse,
        )


class AsyncOutboundVoiceProfilesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOutboundVoiceProfilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOutboundVoiceProfilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOutboundVoiceProfilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncOutboundVoiceProfilesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        billing_group_id: Optional[str] | Omit = omit,
        call_recording: OutboundCallRecordingParam | Omit = omit,
        calling_window: outbound_voice_profile_create_params.CallingWindow | Omit = omit,
        concurrent_call_limit: Optional[int] | Omit = omit,
        daily_spend_limit: str | Omit = omit,
        daily_spend_limit_enabled: bool | Omit = omit,
        enabled: bool | Omit = omit,
        max_destination_rate: float | Omit = omit,
        service_plan: ServicePlan | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        traffic_type: TrafficType | Omit = omit,
        usage_payment_method: UsagePaymentMethod | Omit = omit,
        whitelisted_destinations: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutboundVoiceProfileCreateResponse:
        """
        Create an outbound voice profile.

        Args:
          name: A user-supplied name to help with organization.

          billing_group_id: The ID of the billing group associated with the outbound proflile. Defaults to
              null (for no group assigned).

          calling_window: (BETA) Specifies the time window and call limits for calls made using this
              outbound voice profile. Note that all times are UTC in 24-hour clock time.

          concurrent_call_limit: Must be no more than your global concurrent call limit. Null means no limit.

          daily_spend_limit: The maximum amount of usage charges, in USD, you want Telnyx to allow on this
              outbound voice profile in a day before disallowing new calls.

          daily_spend_limit_enabled: Specifies whether to enforce the daily_spend_limit on this outbound voice
              profile.

          enabled: Specifies whether the outbound voice profile can be used. Disabled profiles will
              result in outbound calls being blocked for the associated Connections.

          max_destination_rate: Maximum rate (price per minute) for a Destination to be allowed when making
              outbound calls.

          service_plan: Indicates the coverage of the termination regions.

          traffic_type: Specifies the type of traffic allowed in this profile.

          usage_payment_method: Setting for how costs for outbound profile are calculated.

          whitelisted_destinations: The list of destinations you want to be able to call using this outbound voice
              profile formatted in alpha2.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/outbound_voice_profiles",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "billing_group_id": billing_group_id,
                    "call_recording": call_recording,
                    "calling_window": calling_window,
                    "concurrent_call_limit": concurrent_call_limit,
                    "daily_spend_limit": daily_spend_limit,
                    "daily_spend_limit_enabled": daily_spend_limit_enabled,
                    "enabled": enabled,
                    "max_destination_rate": max_destination_rate,
                    "service_plan": service_plan,
                    "tags": tags,
                    "traffic_type": traffic_type,
                    "usage_payment_method": usage_payment_method,
                    "whitelisted_destinations": whitelisted_destinations,
                },
                outbound_voice_profile_create_params.OutboundVoiceProfileCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OutboundVoiceProfileCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutboundVoiceProfileRetrieveResponse:
        """
        Retrieves the details of an existing outbound voice profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/outbound_voice_profiles/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OutboundVoiceProfileRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        name: str,
        billing_group_id: Optional[str] | Omit = omit,
        call_recording: OutboundCallRecordingParam | Omit = omit,
        calling_window: outbound_voice_profile_update_params.CallingWindow | Omit = omit,
        concurrent_call_limit: Optional[int] | Omit = omit,
        daily_spend_limit: str | Omit = omit,
        daily_spend_limit_enabled: bool | Omit = omit,
        enabled: bool | Omit = omit,
        max_destination_rate: float | Omit = omit,
        service_plan: ServicePlan | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        traffic_type: TrafficType | Omit = omit,
        usage_payment_method: UsagePaymentMethod | Omit = omit,
        whitelisted_destinations: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutboundVoiceProfileUpdateResponse:
        """
        Updates an existing outbound voice profile.

        Args:
          name: A user-supplied name to help with organization.

          billing_group_id: The ID of the billing group associated with the outbound proflile. Defaults to
              null (for no group assigned).

          calling_window: (BETA) Specifies the time window and call limits for calls made using this
              outbound voice profile.

          concurrent_call_limit: Must be no more than your global concurrent call limit. Null means no limit.

          daily_spend_limit: The maximum amount of usage charges, in USD, you want Telnyx to allow on this
              outbound voice profile in a day before disallowing new calls.

          daily_spend_limit_enabled: Specifies whether to enforce the daily_spend_limit on this outbound voice
              profile.

          enabled: Specifies whether the outbound voice profile can be used. Disabled profiles will
              result in outbound calls being blocked for the associated Connections.

          max_destination_rate: Maximum rate (price per minute) for a Destination to be allowed when making
              outbound calls.

          service_plan: Indicates the coverage of the termination regions.

          traffic_type: Specifies the type of traffic allowed in this profile.

          usage_payment_method: Setting for how costs for outbound profile are calculated.

          whitelisted_destinations: The list of destinations you want to be able to call using this outbound voice
              profile formatted in alpha2.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/outbound_voice_profiles/{id}",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "billing_group_id": billing_group_id,
                    "call_recording": call_recording,
                    "calling_window": calling_window,
                    "concurrent_call_limit": concurrent_call_limit,
                    "daily_spend_limit": daily_spend_limit,
                    "daily_spend_limit_enabled": daily_spend_limit_enabled,
                    "enabled": enabled,
                    "max_destination_rate": max_destination_rate,
                    "service_plan": service_plan,
                    "tags": tags,
                    "traffic_type": traffic_type,
                    "usage_payment_method": usage_payment_method,
                    "whitelisted_destinations": whitelisted_destinations,
                },
                outbound_voice_profile_update_params.OutboundVoiceProfileUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OutboundVoiceProfileUpdateResponse,
        )

    def list(
        self,
        *,
        filter: outbound_voice_profile_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal[
            "enabled",
            "-enabled",
            "created_at",
            "-created_at",
            "name",
            "-name",
            "service_plan",
            "-service_plan",
            "traffic_type",
            "-traffic_type",
            "usage_payment_method",
            "-usage_payment_method",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[OutboundVoiceProfile, AsyncDefaultFlatPagination[OutboundVoiceProfile]]:
        """
        Get all outbound voice profiles belonging to the user that match the given
        filters.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[name][contains]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code>-</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>name</code>: sorts the result by the
                  <code>name</code> field in ascending order.
                </li>

                <li>
                  <code>-name</code>: sorts the result by the
                  <code>name</code> field in descending order.
                </li>
              </ul> <br/>

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/outbound_voice_profiles",
            page=AsyncDefaultFlatPagination[OutboundVoiceProfile],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    outbound_voice_profile_list_params.OutboundVoiceProfileListParams,
                ),
            ),
            model=OutboundVoiceProfile,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OutboundVoiceProfileDeleteResponse:
        """
        Deletes an existing outbound voice profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/outbound_voice_profiles/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OutboundVoiceProfileDeleteResponse,
        )


class OutboundVoiceProfilesResourceWithRawResponse:
    def __init__(self, outbound_voice_profiles: OutboundVoiceProfilesResource) -> None:
        self._outbound_voice_profiles = outbound_voice_profiles

        self.create = to_raw_response_wrapper(
            outbound_voice_profiles.create,
        )
        self.retrieve = to_raw_response_wrapper(
            outbound_voice_profiles.retrieve,
        )
        self.update = to_raw_response_wrapper(
            outbound_voice_profiles.update,
        )
        self.list = to_raw_response_wrapper(
            outbound_voice_profiles.list,
        )
        self.delete = to_raw_response_wrapper(
            outbound_voice_profiles.delete,
        )


class AsyncOutboundVoiceProfilesResourceWithRawResponse:
    def __init__(self, outbound_voice_profiles: AsyncOutboundVoiceProfilesResource) -> None:
        self._outbound_voice_profiles = outbound_voice_profiles

        self.create = async_to_raw_response_wrapper(
            outbound_voice_profiles.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            outbound_voice_profiles.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            outbound_voice_profiles.update,
        )
        self.list = async_to_raw_response_wrapper(
            outbound_voice_profiles.list,
        )
        self.delete = async_to_raw_response_wrapper(
            outbound_voice_profiles.delete,
        )


class OutboundVoiceProfilesResourceWithStreamingResponse:
    def __init__(self, outbound_voice_profiles: OutboundVoiceProfilesResource) -> None:
        self._outbound_voice_profiles = outbound_voice_profiles

        self.create = to_streamed_response_wrapper(
            outbound_voice_profiles.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            outbound_voice_profiles.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            outbound_voice_profiles.update,
        )
        self.list = to_streamed_response_wrapper(
            outbound_voice_profiles.list,
        )
        self.delete = to_streamed_response_wrapper(
            outbound_voice_profiles.delete,
        )


class AsyncOutboundVoiceProfilesResourceWithStreamingResponse:
    def __init__(self, outbound_voice_profiles: AsyncOutboundVoiceProfilesResource) -> None:
        self._outbound_voice_profiles = outbound_voice_profiles

        self.create = async_to_streamed_response_wrapper(
            outbound_voice_profiles.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            outbound_voice_profiles.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            outbound_voice_profiles.update,
        )
        self.list = async_to_streamed_response_wrapper(
            outbound_voice_profiles.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            outbound_voice_profiles.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "UsersResource",
    "AsyncUsersResource",
    "UsersResourceWithRawResponse",
    "AsyncUsersResourceWithRawResponse",
    "UsersResourceWithStreamingResponse",
    "AsyncUsersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .oauth_client import OAuthClient

__all__ = ["OAuthClientUpdateResponse"]


class OAuthClientUpdateResponse(BaseModel):
    data: Optional[OAuthClient] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .agents import (
    AgentsResource,
    AsyncAgentsResource,
    AgentsResourceWithRawResponse,
    AsyncAgentsResourceWithRawResponse,
    AgentsResourceWithStreamingResponse,
    AsyncAgentsResourceWithStreamingResponse,
)
from ...._types import Body, Query, Headers, NotGiven, SequenceNotStr, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.messaging import rc_list_bulk_capabilities_params
from ....types.messaging.rc_invite_test_number_response import RcInviteTestNumberResponse
from ....types.messaging.rc_retrieve_capabilities_response import RcRetrieveCapabilitiesResponse
from ....types.messaging.rc_list_bulk_capabilities_response import RcListBulkCapabilitiesResponse

__all__ = ["RcsResource", "AsyncRcsResource"]


class RcsResource(SyncAPIResource):
    @cached_property
    def agents(self) -> AgentsResource:
        return AgentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> RcsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return RcsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RcsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return RcsResourceWithStreamingResponse(self)

    def invite_test_number(
        self,
        phone_number: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RcInviteTestNumberResponse:
        """
        Adds a test phone number to an RCS agent for testing purposes.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._put(
            f"/messaging/rcs/test_number_invite/{id}/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RcInviteTestNumberResponse,
        )

    def list_bulk_capabilities(
        self,
        *,
        agent_id: str,
        phone_numbers: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RcListBulkCapabilitiesResponse:
        """
        Check RCS capabilities (batch)

        Args:
          agent_id: RCS Agent ID

          phone_numbers: List of phone numbers to check

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/messaging/rcs/bulk_capabilities",
            body=maybe_transform(
                {
                    "agent_id": agent_id,
                    "phone_numbers": phone_numbers,
                },
                rc_list_bulk_capabilities_params.RcListBulkCapabilitiesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RcListBulkCapabilitiesResponse,
        )

    def retrieve_capabilities(
        self,
        phone_number: str,
        *,
        agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RcRetrieveCapabilitiesResponse:
        """
        Check RCS capabilities

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._get(
            f"/messaging/rcs/capabilities/{agent_id}/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RcRetrieveCapabilitiesResponse,
        )


class AsyncRcsResource(AsyncAPIResource):
    @cached_property
    def agents(self) -> AsyncAgentsResource:
        return AsyncAgentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRcsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRcsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRcsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncRcsResourceWithStreamingResponse(self)

    async def invite_test_number(
        self,
        phone_number: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RcInviteTestNumberResponse:
        """
        Adds a test phone number to an RCS agent for testing purposes.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._put(
            f"/messaging/rcs/test_number_invite/{id}/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RcInviteTestNumberResponse,
        )

    async def list_bulk_capabilities(
        self,
        *,
        agent_id: str,
        phone_numbers: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RcListBulkCapabilitiesResponse:
        """
        Check RCS capabilities (batch)

        Args:
          agent_id: RCS Agent ID

          phone_numbers: List of phone numbers to check

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/messaging/rcs/bulk_capabilities",
            body=await async_maybe_transform(
                {
                    "agent_id": agent_id,
                    "phone_numbers": phone_numbers,
                },
                rc_list_bulk_capabilities_params.RcListBulkCapabilitiesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RcListBulkCapabilitiesResponse,
        )

    async def retrieve_capabilities(
        self,
        phone_number: str,
        *,
        agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RcRetrieveCapabilitiesResponse:
        """
        Check RCS capabilities

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._get(
            f"/messaging/rcs/capabilities/{agent_id}/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RcRetrieveCapabilitiesResponse,
        )


class RcsResourceWithRawResponse:
    def __init__(self, rcs: RcsResource) -> None:
        self._rcs = rcs

        self.invite_test_number = to_raw_response_wrapper(
            rcs.invite_test_number,
        )
        self.list_bulk_capabilities = to_raw_response_wrapper(
            rcs.list_bulk_capabilities,
        )
        self.retrieve_capabilities = to_raw_response_wrapper(
            rcs.retrieve_capabilities,
        )

    @cached_property
    def agents(self) -> AgentsResourceWithRawResponse:
        return AgentsResourceWithRawResponse(self._rcs.agents)


class AsyncRcsResourceWithRawResponse:
    def __init__(self, rcs: AsyncRcsResource) -> None:
        self._rcs = rcs

        self.invite_test_number = async_to_raw_response_wrapper(
            rcs.invite_test_number,
        )
        self.list_bulk_capabilities = async_to_raw_response_wrapper(
            rcs.list_bulk_capabilities,
        )
        self.retrieve_capabilities = async_to_raw_response_wrapper(
            rcs.retrieve_capabilities,
        )

    @cached_property
    def agents(self) -> AsyncAgentsResourceWithRawResponse:
        return AsyncAgentsResourceWithRawResponse(self._rcs.agents)


class RcsResourceWithStreamingResponse:
    def __init__(self, rcs: RcsResource) -> None:
        self._rcs = rcs

        self.invite_test_number = to_streamed_response_wrapper(
            rcs.invite_test_number,
        )
        self.list_bulk_capabilities = to_streamed_response_wrapper(
            rcs.list_bulk_capabilities,
        )
        self.retrieve_capabilities = to_streamed_response_wrapper(
            rcs.retrieve_capabilities,
        )

    @cached_property
    def agents(self) -> AgentsResourceWithStreamingResponse:
        return AgentsResourceWithStreamingResponse(self._rcs.agents)


class AsyncRcsResourceWithStreamingResponse:
    def __init__(self, rcs: AsyncRcsResource) -> None:
        self._rcs = rcs

        self.invite_test_number = async_to_streamed_response_wrapper(
            rcs.invite_test_number,
        )
        self.list_bulk_capabilities = async_to_streamed_response_wrapper(
            rcs.list_bulk_capabilities,
        )
        self.retrieve_capabilities = async_to_streamed_response_wrapper(
            rcs.retrieve_capabilities,
        )

    @cached_property
    def agents(self) -> AsyncAgentsResourceWithStreamingResponse:
        return AsyncAgentsResourceWithStreamingResponse(self._rcs.agents)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import TypedDict

__all__ = ["ConversationCreateParams"]


class ConversationCreateParams(TypedDict, total=False):
    metadata: Dict[str, str]
    """Metadata associated with the conversation."""

    name: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.texml import call_update_params, call_initiate_params
from ..._base_client import make_request_options
from ...types.texml.call_update_response import CallUpdateResponse
from ...types.texml.call_initiate_response import CallInitiateResponse

__all__ = ["CallsResource", "AsyncCallsResource"]


class CallsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CallsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CallsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CallsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CallsResourceWithStreamingResponse(self)

    def update(
        self,
        call_sid: str,
        *,
        fallback_method: Literal["GET", "POST"] | Omit = omit,
        fallback_url: str | Omit = omit,
        method: Literal["GET", "POST"] | Omit = omit,
        status: str | Omit = omit,
        status_callback: str | Omit = omit,
        status_callback_method: Literal["GET", "POST"] | Omit = omit,
        texml: str | Omit = omit,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallUpdateResponse:
        """Update TeXML call.

        Please note that the keys present in the payload MUST BE
        formatted in CamelCase as specified in the example.

        Args:
          fallback_method: HTTP request type used for `FallbackUrl`.

          fallback_url: A failover URL for which Telnyx will retrieve the TeXML call instructions if the
              Url is not responding.

          method: HTTP request type used for `Url`.

          status: The value to set the call status to. Setting the status to completed ends the
              call.

          status_callback: URL destination for Telnyx to send status callback events to for the call.

          status_callback_method: HTTP request type used for `StatusCallback`.

          texml: TeXML to replace the current one with.

          url: The URL where TeXML will make a request to retrieve a new set of TeXML
              instructions to continue the call flow.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not call_sid:
            raise ValueError(f"Expected a non-empty value for `call_sid` but received {call_sid!r}")
        return self._post(
            f"/texml/calls/{call_sid}/update",
            body=maybe_transform(
                {
                    "fallback_method": fallback_method,
                    "fallback_url": fallback_url,
                    "method": method,
                    "status": status,
                    "status_callback": status_callback,
                    "status_callback_method": status_callback_method,
                    "texml": texml,
                    "url": url,
                },
                call_update_params.CallUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallUpdateResponse,
        )

    def initiate(
        self,
        application_id: str,
        *,
        from_: str,
        to: str,
        async_amd: bool | Omit = omit,
        async_amd_status_callback: str | Omit = omit,
        async_amd_status_callback_method: Literal["GET", "POST"] | Omit = omit,
        caller_id: str | Omit = omit,
        cancel_playback_on_detect_message_end: bool | Omit = omit,
        cancel_playback_on_machine_detection: bool | Omit = omit,
        detection_mode: Literal["Premium", "Regular"] | Omit = omit,
        fallback_url: str | Omit = omit,
        machine_detection: Literal["Enable", "Disable", "DetectMessageEnd"] | Omit = omit,
        machine_detection_silence_timeout: int | Omit = omit,
        machine_detection_speech_end_threshold: int | Omit = omit,
        machine_detection_speech_threshold: int | Omit = omit,
        machine_detection_timeout: int | Omit = omit,
        preferred_codecs: str | Omit = omit,
        record: bool | Omit = omit,
        recording_channels: Literal["mono", "dual"] | Omit = omit,
        recording_status_callback: str | Omit = omit,
        recording_status_callback_event: str | Omit = omit,
        recording_status_callback_method: Literal["GET", "POST"] | Omit = omit,
        recording_timeout: int | Omit = omit,
        recording_track: Literal["inbound", "outbound", "both"] | Omit = omit,
        sip_auth_password: str | Omit = omit,
        sip_auth_username: str | Omit = omit,
        status_callback: str | Omit = omit,
        status_callback_event: Literal["initiated", "ringing", "answered", "completed"] | Omit = omit,
        status_callback_method: Literal["GET", "POST"] | Omit = omit,
        trim: Literal["trim-silence", "do-not-trim"] | Omit = omit,
        url: str | Omit = omit,
        url_method: Literal["GET", "POST"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallInitiateResponse:
        """Initiate an outbound TeXML call.

        Telnyx will request TeXML from the XML Request
        URL configured for the connection in the Mission Control Portal.

        Args:
          from_: The phone number of the party that initiated the call. Phone numbers are
              formatted with a `+` and country code.

          to: The phone number of the called party. Phone numbers are formatted with a `+` and
              country code.

          async_amd: Select whether to perform answering machine detection in the background. By
              default execution is blocked until Answering Machine Detection is completed.

          async_amd_status_callback: URL destination for Telnyx to send AMD callback events to for the call.

          async_amd_status_callback_method: HTTP request type used for `AsyncAmdStatusCallback`. The default value is
              inherited from TeXML Application setting.

          caller_id: To be used as the caller id name (SIP From Display Name) presented to the
              destination (`To` number). The string should have a maximum of 128 characters,
              containing only letters, numbers, spaces, and `-_~!.+` special characters. If
              ommited, the display name will be the same as the number in the `From` field.

          cancel_playback_on_detect_message_end: Whether to cancel ongoing playback on `greeting ended` detection. Defaults to
              `true`.

          cancel_playback_on_machine_detection: Whether to cancel ongoing playback on `machine` detection. Defaults to `true`.

          detection_mode: Allows you to chose between Premium and Standard detections.

          fallback_url: A failover URL for which Telnyx will retrieve the TeXML call instructions if the
              `Url` is not responding.

          machine_detection: Enables Answering Machine Detection.

          machine_detection_silence_timeout: If initial silence duration is greater than this value, consider it a machine.
              Ignored when `premium` detection is used.

          machine_detection_speech_end_threshold: Silence duration threshold after a greeting message or voice for it be
              considered human. Ignored when `premium` detection is used.

          machine_detection_speech_threshold: Maximum threshold of a human greeting. If greeting longer than this value,
              considered machine. Ignored when `premium` detection is used.

          machine_detection_timeout: Maximum timeout threshold in milliseconds for overall detection.

          preferred_codecs: The list of comma-separated codecs to be offered on a call.

          record: Whether to record the entire participant's call leg. Defaults to `false`.

          recording_channels: The number of channels in the final recording. Defaults to `mono`.

          recording_status_callback: The URL the recording callbacks will be sent to.

          recording_status_callback_event: The changes to the recording's state that should generate a call to
              `RecoridngStatusCallback`. Can be: `in-progress`, `completed` and `absent`.
              Separate multiple values with a space. Defaults to `completed`.

          recording_status_callback_method: HTTP request type used for `RecordingStatusCallback`. Defaults to `POST`.

          recording_timeout: The number of seconds that Telnyx will wait for the recording to be stopped if
              silence is detected. The timer only starts when the speech is detected. Please
              note that the transcription is used to detect silence and the related charge
              will be applied. The minimum value is 0. The default value is 0 (infinite)

          recording_track: The audio track to record for the call. The default is `both`.

          sip_auth_password: The password to use for SIP authentication.

          sip_auth_username: The username to use for SIP authentication.

          status_callback: URL destination for Telnyx to send status callback events to for the call.

          status_callback_event: The call events for which Telnyx should send a webhook. Multiple events can be
              defined when separated by a space.

          status_callback_method: HTTP request type used for `StatusCallback`.

          trim: Whether to trim any leading and trailing silence from the recording. Defaults to
              `trim-silence`.

          url: The URL from which Telnyx will retrieve the TeXML call instructions.

          url_method: HTTP request type used for `Url`. The default value is inherited from TeXML
              Application setting.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return self._post(
            f"/texml/calls/{application_id}",
            body=maybe_transform(
                {
                    "from_": from_,
                    "to": to,
                    "async_amd": async_amd,
                    "async_amd_status_callback": async_amd_status_callback,
                    "async_amd_status_callback_method": async_amd_status_callback_method,
                    "caller_id": caller_id,
                    "cancel_playback_on_detect_message_end": cancel_playback_on_detect_message_end,
                    "cancel_playback_on_machine_detection": cancel_playback_on_machine_detection,
                    "detection_mode": detection_mode,
                    "fallback_url": fallback_url,
                    "machine_detection": machine_detection,
                    "machine_detection_silence_timeout": machine_detection_silence_timeout,
                    "machine_detection_speech_end_threshold": machine_detection_speech_end_threshold,
                    "machine_detection_speech_threshold": machine_detection_speech_threshold,
                    "machine_detection_timeout": machine_detection_timeout,
                    "preferred_codecs": preferred_codecs,
                    "record": record,
                    "recording_channels": recording_channels,
                    "recording_status_callback": recording_status_callback,
                    "recording_status_callback_event": recording_status_callback_event,
                    "recording_status_callback_method": recording_status_callback_method,
                    "recording_timeout": recording_timeout,
                    "recording_track": recording_track,
                    "sip_auth_password": sip_auth_password,
                    "sip_auth_username": sip_auth_username,
                    "status_callback": status_callback,
                    "status_callback_event": status_callback_event,
                    "status_callback_method": status_callback_method,
                    "trim": trim,
                    "url": url,
                    "url_method": url_method,
                },
                call_initiate_params.CallInitiateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallInitiateResponse,
        )


class AsyncCallsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCallsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCallsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCallsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCallsResourceWithStreamingResponse(self)

    async def update(
        self,
        call_sid: str,
        *,
        fallback_method: Literal["GET", "POST"] | Omit = omit,
        fallback_url: str | Omit = omit,
        method: Literal["GET", "POST"] | Omit = omit,
        status: str | Omit = omit,
        status_callback: str | Omit = omit,
        status_callback_method: Literal["GET", "POST"] | Omit = omit,
        texml: str | Omit = omit,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallUpdateResponse:
        """Update TeXML call.

        Please note that the keys present in the payload MUST BE
        formatted in CamelCase as specified in the example.

        Args:
          fallback_method: HTTP request type used for `FallbackUrl`.

          fallback_url: A failover URL for which Telnyx will retrieve the TeXML call instructions if the
              Url is not responding.

          method: HTTP request type used for `Url`.

          status: The value to set the call status to. Setting the status to completed ends the
              call.

          status_callback: URL destination for Telnyx to send status callback events to for the call.

          status_callback_method: HTTP request type used for `StatusCallback`.

          texml: TeXML to replace the current one with.

          url: The URL where TeXML will make a request to retrieve a new set of TeXML
              instructions to continue the call flow.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not call_sid:
            raise ValueError(f"Expected a non-empty value for `call_sid` but received {call_sid!r}")
        return await self._post(
            f"/texml/calls/{call_sid}/update",
            body=await async_maybe_transform(
                {
                    "fallback_method": fallback_method,
                    "fallback_url": fallback_url,
                    "method": method,
                    "status": status,
                    "status_callback": status_callback,
                    "status_callback_method": status_callback_method,
                    "texml": texml,
                    "url": url,
                },
                call_update_params.CallUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallUpdateResponse,
        )

    async def initiate(
        self,
        application_id: str,
        *,
        from_: str,
        to: str,
        async_amd: bool | Omit = omit,
        async_amd_status_callback: str | Omit = omit,
        async_amd_status_callback_method: Literal["GET", "POST"] | Omit = omit,
        caller_id: str | Omit = omit,
        cancel_playback_on_detect_message_end: bool | Omit = omit,
        cancel_playback_on_machine_detection: bool | Omit = omit,
        detection_mode: Literal["Premium", "Regular"] | Omit = omit,
        fallback_url: str | Omit = omit,
        machine_detection: Literal["Enable", "Disable", "DetectMessageEnd"] | Omit = omit,
        machine_detection_silence_timeout: int | Omit = omit,
        machine_detection_speech_end_threshold: int | Omit = omit,
        machine_detection_speech_threshold: int | Omit = omit,
        machine_detection_timeout: int | Omit = omit,
        preferred_codecs: str | Omit = omit,
        record: bool | Omit = omit,
        recording_channels: Literal["mono", "dual"] | Omit = omit,
        recording_status_callback: str | Omit = omit,
        recording_status_callback_event: str | Omit = omit,
        recording_status_callback_method: Literal["GET", "POST"] | Omit = omit,
        recording_timeout: int | Omit = omit,
        recording_track: Literal["inbound", "outbound", "both"] | Omit = omit,
        sip_auth_password: str | Omit = omit,
        sip_auth_username: str | Omit = omit,
        status_callback: str | Omit = omit,
        status_callback_event: Literal["initiated", "ringing", "answered", "completed"] | Omit = omit,
        status_callback_method: Literal["GET", "POST"] | Omit = omit,
        trim: Literal["trim-silence", "do-not-trim"] | Omit = omit,
        url: str | Omit = omit,
        url_method: Literal["GET", "POST"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallInitiateResponse:
        """Initiate an outbound TeXML call.

        Telnyx will request TeXML from the XML Request
        URL configured for the connection in the Mission Control Portal.

        Args:
          from_: The phone number of the party that initiated the call. Phone numbers are
              formatted with a `+` and country code.

          to: The phone number of the called party. Phone numbers are formatted with a `+` and
              country code.

          async_amd: Select whether to perform answering machine detection in the background. By
              default execution is blocked until Answering Machine Detection is completed.

          async_amd_status_callback: URL destination for Telnyx to send AMD callback events to for the call.

          async_amd_status_callback_method: HTTP request type used for `AsyncAmdStatusCallback`. The default value is
              inherited from TeXML Application setting.

          caller_id: To be used as the caller id name (SIP From Display Name) presented to the
              destination (`To` number). The string should have a maximum of 128 characters,
              containing only letters, numbers, spaces, and `-_~!.+` special characters. If
              ommited, the display name will be the same as the number in the `From` field.

          cancel_playback_on_detect_message_end: Whether to cancel ongoing playback on `greeting ended` detection. Defaults to
              `true`.

          cancel_playback_on_machine_detection: Whether to cancel ongoing playback on `machine` detection. Defaults to `true`.

          detection_mode: Allows you to chose between Premium and Standard detections.

          fallback_url: A failover URL for which Telnyx will retrieve the TeXML call instructions if the
              `Url` is not responding.

          machine_detection: Enables Answering Machine Detection.

          machine_detection_silence_timeout: If initial silence duration is greater than this value, consider it a machine.
              Ignored when `premium` detection is used.

          machine_detection_speech_end_threshold: Silence duration threshold after a greeting message or voice for it be
              considered human. Ignored when `premium` detection is used.

          machine_detection_speech_threshold: Maximum threshold of a human greeting. If greeting longer than this value,
              considered machine. Ignored when `premium` detection is used.

          machine_detection_timeout: Maximum timeout threshold in milliseconds for overall detection.

          preferred_codecs: The list of comma-separated codecs to be offered on a call.

          record: Whether to record the entire participant's call leg. Defaults to `false`.

          recording_channels: The number of channels in the final recording. Defaults to `mono`.

          recording_status_callback: The URL the recording callbacks will be sent to.

          recording_status_callback_event: The changes to the recording's state that should generate a call to
              `RecoridngStatusCallback`. Can be: `in-progress`, `completed` and `absent`.
              Separate multiple values with a space. Defaults to `completed`.

          recording_status_callback_method: HTTP request type used for `RecordingStatusCallback`. Defaults to `POST`.

          recording_timeout: The number of seconds that Telnyx will wait for the recording to be stopped if
              silence is detected. The timer only starts when the speech is detected. Please
              note that the transcription is used to detect silence and the related charge
              will be applied. The minimum value is 0. The default value is 0 (infinite)

          recording_track: The audio track to record for the call. The default is `both`.

          sip_auth_password: The password to use for SIP authentication.

          sip_auth_username: The username to use for SIP authentication.

          status_callback: URL destination for Telnyx to send status callback events to for the call.

          status_callback_event: The call events for which Telnyx should send a webhook. Multiple events can be
              defined when separated by a space.

          status_callback_method: HTTP request type used for `StatusCallback`.

          trim: Whether to trim any leading and trailing silence from the recording. Defaults to
              `trim-silence`.

          url: The URL from which Telnyx will retrieve the TeXML call instructions.

          url_method: HTTP request type used for `Url`. The default value is inherited from TeXML
              Application setting.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return await self._post(
            f"/texml/calls/{application_id}",
            body=await async_maybe_transform(
                {
                    "from_": from_,
                    "to": to,
                    "async_amd": async_amd,
                    "async_amd_status_callback": async_amd_status_callback,
                    "async_amd_status_callback_method": async_amd_status_callback_method,
                    "caller_id": caller_id,
                    "cancel_playback_on_detect_message_end": cancel_playback_on_detect_message_end,
                    "cancel_playback_on_machine_detection": cancel_playback_on_machine_detection,
                    "detection_mode": detection_mode,
                    "fallback_url": fallback_url,
                    "machine_detection": machine_detection,
                    "machine_detection_silence_timeout": machine_detection_silence_timeout,
                    "machine_detection_speech_end_threshold": machine_detection_speech_end_threshold,
                    "machine_detection_speech_threshold": machine_detection_speech_threshold,
                    "machine_detection_timeout": machine_detection_timeout,
                    "preferred_codecs": preferred_codecs,
                    "record": record,
                    "recording_channels": recording_channels,
                    "recording_status_callback": recording_status_callback,
                    "recording_status_callback_event": recording_status_callback_event,
                    "recording_status_callback_method": recording_status_callback_method,
                    "recording_timeout": recording_timeout,
                    "recording_track": recording_track,
                    "sip_auth_password": sip_auth_password,
                    "sip_auth_username": sip_auth_username,
                    "status_callback": status_callback,
                    "status_callback_event": status_callback_event,
                    "status_callback_method": status_callback_method,
                    "trim": trim,
                    "url": url,
                    "url_method": url_method,
                },
                call_initiate_params.CallInitiateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallInitiateResponse,
        )


class CallsResourceWithRawResponse:
    def __init__(self, calls: CallsResource) -> None:
        self._calls = calls

        self.update = to_raw_response_wrapper(
            calls.update,
        )
        self.initiate = to_raw_response_wrapper(
            calls.initiate,
        )


class AsyncCallsResourceWithRawResponse:
    def __init__(self, calls: AsyncCallsResource) -> None:
        self._calls = calls

        self.update = async_to_raw_response_wrapper(
            calls.update,
        )
        self.initiate = async_to_raw_response_wrapper(
            calls.initiate,
        )


class CallsResourceWithStreamingResponse:
    def __init__(self, calls: CallsResource) -> None:
        self._calls = calls

        self.update = to_streamed_response_wrapper(
            calls.update,
        )
        self.initiate = to_streamed_response_wrapper(
            calls.initiate,
        )


class AsyncCallsResourceWithStreamingResponse:
    def __init__(self, calls: AsyncCallsResource) -> None:
        self._calls = calls

        self.update = async_to_streamed_response_wrapper(
            calls.update,
        )
        self.initiate = async_to_streamed_response_wrapper(
            calls.initiate,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ReleaseRetrieveResponse", "Data", "DataTelephoneNumber"]


class DataTelephoneNumber(BaseModel):
    number_id: Optional[str] = None
    """Phone number ID from the Telnyx API."""

    phone_number: Optional[str] = None
    """Phone number in E164 format."""


class Data(BaseModel):
    created_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    error_message: Optional[str] = None
    """A message set if there is an error with the upload process."""

    status: Optional[
        Literal["pending_upload", "pending", "in_progress", "complete", "failed", "expired", "unknown"]
    ] = None
    """Represents the status of the release on Microsoft Teams."""

    telephone_numbers: Optional[List[DataTelephoneNumber]] = None

    tenant_id: Optional[str] = None

    ticket_id: Optional[str] = None
    """Uniquely identifies the resource."""


class ReleaseRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..types import (
    DtmfType,
    EncryptedMedia,
    AnchorsiteOverride,
    ip_connection_list_params,
    ip_connection_create_params,
    ip_connection_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.dtmf_type import DtmfType
from ..types.ip_connection import IPConnection
from ..types.encrypted_media import EncryptedMedia
from ..types.inbound_ip_param import InboundIPParam
from ..types.outbound_ip_param import OutboundIPParam
from ..types.anchorsite_override import AnchorsiteOverride
from ..types.ip_connection_create_response import IPConnectionCreateResponse
from ..types.ip_connection_delete_response import IPConnectionDeleteResponse
from ..types.ip_connection_update_response import IPConnectionUpdateResponse
from ..types.connection_rtcp_settings_param import ConnectionRtcpSettingsParam
from ..types.ip_connection_retrieve_response import IPConnectionRetrieveResponse

__all__ = ["IPConnectionsResource", "AsyncIPConnectionsResource"]


class IPConnectionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IPConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return IPConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IPConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return IPConnectionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        active: bool | Omit = omit,
        anchorsite_override: AnchorsiteOverride | Omit = omit,
        android_push_credential_id: Optional[str] | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        connection_name: str | Omit = omit,
        default_on_hold_comfort_noise_enabled: bool | Omit = omit,
        dtmf_type: DtmfType | Omit = omit,
        encode_contact_header_enabled: bool | Omit = omit,
        encrypted_media: Optional[EncryptedMedia] | Omit = omit,
        inbound: ip_connection_create_params.Inbound | Omit = omit,
        ios_push_credential_id: Optional[str] | Omit = omit,
        noise_suppression: Literal["inbound", "outbound", "both", "disabled"] | Omit = omit,
        noise_suppression_details: ip_connection_create_params.NoiseSuppressionDetails | Omit = omit,
        onnet_t38_passthrough_enabled: bool | Omit = omit,
        outbound: OutboundIPParam | Omit = omit,
        rtcp_settings: ConnectionRtcpSettingsParam | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        transport_protocol: Literal["UDP", "TCP", "TLS"] | Omit = omit,
        webhook_api_version: Literal["1", "2"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPConnectionCreateResponse:
        """
        Creates an IP connection.

        Args:
          active: Defaults to true

          anchorsite_override: `Latency` directs Telnyx to route media through the site with the lowest
              round-trip time to the user's connection. Telnyx calculates this time using ICMP
              ping messages. This can be disabled by specifying a site to handle all media.

          android_push_credential_id: The uuid of the push credential for Android

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this connection.

          default_on_hold_comfort_noise_enabled: When enabled, Telnyx will generate comfort noise when you place the call on
              hold. If disabled, you will need to generate comfort noise or on hold music to
              avoid RTP timeout.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          encode_contact_header_enabled: Encode the SIP contact header sent by Telnyx to avoid issues for NAT or ALG
              scenarios.

          encrypted_media: Enable use of SRTP for encryption. Cannot be set if the transport_portocol is
              TLS.

          ios_push_credential_id: The uuid of the push credential for Ios

          noise_suppression: Controls when noise suppression is applied to calls. When set to 'inbound',
              noise suppression is applied to incoming audio. When set to 'outbound', it's
              applied to outgoing audio. When set to 'both', it's applied in both directions.
              When set to 'disabled', noise suppression is turned off.

          noise_suppression_details: Configuration options for noise suppression. These settings are stored
              regardless of the noise_suppression value, but only take effect when
              noise_suppression is not 'disabled'. If you disable noise suppression and later
              re-enable it, the previously configured settings will be used.

          onnet_t38_passthrough_enabled: Enable on-net T38 if you prefer the sender and receiver negotiating T38 directly
              if both are on the Telnyx network. If this is disabled, Telnyx will be able to
              use T38 on just one leg of the call depending on each leg's settings.

          tags: Tags associated with the connection.

          transport_protocol: One of UDP, TLS, or TCP. Applies only to connections with IP authentication or
              FQDN authentication.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1 or v2.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/ip_connections",
            body=maybe_transform(
                {
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "android_push_credential_id": android_push_credential_id,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "connection_name": connection_name,
                    "default_on_hold_comfort_noise_enabled": default_on_hold_comfort_noise_enabled,
                    "dtmf_type": dtmf_type,
                    "encode_contact_header_enabled": encode_contact_header_enabled,
                    "encrypted_media": encrypted_media,
                    "inbound": inbound,
                    "ios_push_credential_id": ios_push_credential_id,
                    "noise_suppression": noise_suppression,
                    "noise_suppression_details": noise_suppression_details,
                    "onnet_t38_passthrough_enabled": onnet_t38_passthrough_enabled,
                    "outbound": outbound,
                    "rtcp_settings": rtcp_settings,
                    "tags": tags,
                    "transport_protocol": transport_protocol,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                ip_connection_create_params.IPConnectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPConnectionCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPConnectionRetrieveResponse:
        """
        Retrieves the details of an existing ip connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/ip_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPConnectionRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        active: bool | Omit = omit,
        anchorsite_override: AnchorsiteOverride | Omit = omit,
        android_push_credential_id: Optional[str] | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        connection_name: str | Omit = omit,
        default_on_hold_comfort_noise_enabled: bool | Omit = omit,
        dtmf_type: DtmfType | Omit = omit,
        encode_contact_header_enabled: bool | Omit = omit,
        encrypted_media: Optional[EncryptedMedia] | Omit = omit,
        inbound: InboundIPParam | Omit = omit,
        ios_push_credential_id: Optional[str] | Omit = omit,
        noise_suppression: Literal["inbound", "outbound", "both", "disabled"] | Omit = omit,
        noise_suppression_details: ip_connection_update_params.NoiseSuppressionDetails | Omit = omit,
        onnet_t38_passthrough_enabled: bool | Omit = omit,
        outbound: OutboundIPParam | Omit = omit,
        rtcp_settings: ConnectionRtcpSettingsParam | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        transport_protocol: Literal["UDP", "TCP", "TLS"] | Omit = omit,
        webhook_api_version: Literal["1", "2"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPConnectionUpdateResponse:
        """
        Updates settings of an existing IP connection.

        Args:
          active: Defaults to true

          anchorsite_override: `Latency` directs Telnyx to route media through the site with the lowest
              round-trip time to the user's connection. Telnyx calculates this time using ICMP
              ping messages. This can be disabled by specifying a site to handle all media.

          android_push_credential_id: The uuid of the push credential for Android

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this connection.

          default_on_hold_comfort_noise_enabled: When enabled, Telnyx will generate comfort noise when you place the call on
              hold. If disabled, you will need to generate comfort noise or on hold music to
              avoid RTP timeout.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          encode_contact_header_enabled: Encode the SIP contact header sent by Telnyx to avoid issues for NAT or ALG
              scenarios.

          encrypted_media: Enable use of SRTP for encryption. Cannot be set if the transport_portocol is
              TLS.

          ios_push_credential_id: The uuid of the push credential for Ios

          noise_suppression: Controls when noise suppression is applied to calls. When set to 'inbound',
              noise suppression is applied to incoming audio. When set to 'outbound', it's
              applied to outgoing audio. When set to 'both', it's applied in both directions.
              When set to 'disabled', noise suppression is turned off.

          noise_suppression_details: Configuration options for noise suppression. These settings are stored
              regardless of the noise_suppression value, but only take effect when
              noise_suppression is not 'disabled'. If you disable noise suppression and later
              re-enable it, the previously configured settings will be used.

          onnet_t38_passthrough_enabled: Enable on-net T38 if you prefer the sender and receiver negotiating T38 directly
              if both are on the Telnyx network. If this is disabled, Telnyx will be able to
              use T38 on just one leg of the call depending on each leg's settings.

          tags: Tags associated with the connection.

          transport_protocol: One of UDP, TLS, or TCP. Applies only to connections with IP authentication or
              FQDN authentication.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1 or v2.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/ip_connections/{id}",
            body=maybe_transform(
                {
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "android_push_credential_id": android_push_credential_id,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "connection_name": connection_name,
                    "default_on_hold_comfort_noise_enabled": default_on_hold_comfort_noise_enabled,
                    "dtmf_type": dtmf_type,
                    "encode_contact_header_enabled": encode_contact_header_enabled,
                    "encrypted_media": encrypted_media,
                    "inbound": inbound,
                    "ios_push_credential_id": ios_push_credential_id,
                    "noise_suppression": noise_suppression,
                    "noise_suppression_details": noise_suppression_details,
                    "onnet_t38_passthrough_enabled": onnet_t38_passthrough_enabled,
                    "outbound": outbound,
                    "rtcp_settings": rtcp_settings,
                    "tags": tags,
                    "transport_protocol": transport_protocol,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                ip_connection_update_params.IPConnectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPConnectionUpdateResponse,
        )

    def list(
        self,
        *,
        filter: ip_connection_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["created_at", "connection_name", "active"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[IPConnection]:
        """
        Returns a list of your IP connections.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[connection_name], filter[fqdn], filter[outbound_voice_profile_id],
              filter[outbound.outbound_voice_profile_id]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in ascending order.
                </li>

                <li>
                  <code>-connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/ip_connections",
            page=SyncDefaultFlatPagination[IPConnection],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    ip_connection_list_params.IPConnectionListParams,
                ),
            ),
            model=IPConnection,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPConnectionDeleteResponse:
        """
        Deletes an existing IP connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/ip_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPConnectionDeleteResponse,
        )


class AsyncIPConnectionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIPConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIPConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIPConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncIPConnectionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        active: bool | Omit = omit,
        anchorsite_override: AnchorsiteOverride | Omit = omit,
        android_push_credential_id: Optional[str] | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        connection_name: str | Omit = omit,
        default_on_hold_comfort_noise_enabled: bool | Omit = omit,
        dtmf_type: DtmfType | Omit = omit,
        encode_contact_header_enabled: bool | Omit = omit,
        encrypted_media: Optional[EncryptedMedia] | Omit = omit,
        inbound: ip_connection_create_params.Inbound | Omit = omit,
        ios_push_credential_id: Optional[str] | Omit = omit,
        noise_suppression: Literal["inbound", "outbound", "both", "disabled"] | Omit = omit,
        noise_suppression_details: ip_connection_create_params.NoiseSuppressionDetails | Omit = omit,
        onnet_t38_passthrough_enabled: bool | Omit = omit,
        outbound: OutboundIPParam | Omit = omit,
        rtcp_settings: ConnectionRtcpSettingsParam | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        transport_protocol: Literal["UDP", "TCP", "TLS"] | Omit = omit,
        webhook_api_version: Literal["1", "2"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPConnectionCreateResponse:
        """
        Creates an IP connection.

        Args:
          active: Defaults to true

          anchorsite_override: `Latency` directs Telnyx to route media through the site with the lowest
              round-trip time to the user's connection. Telnyx calculates this time using ICMP
              ping messages. This can be disabled by specifying a site to handle all media.

          android_push_credential_id: The uuid of the push credential for Android

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this connection.

          default_on_hold_comfort_noise_enabled: When enabled, Telnyx will generate comfort noise when you place the call on
              hold. If disabled, you will need to generate comfort noise or on hold music to
              avoid RTP timeout.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          encode_contact_header_enabled: Encode the SIP contact header sent by Telnyx to avoid issues for NAT or ALG
              scenarios.

          encrypted_media: Enable use of SRTP for encryption. Cannot be set if the transport_portocol is
              TLS.

          ios_push_credential_id: The uuid of the push credential for Ios

          noise_suppression: Controls when noise suppression is applied to calls. When set to 'inbound',
              noise suppression is applied to incoming audio. When set to 'outbound', it's
              applied to outgoing audio. When set to 'both', it's applied in both directions.
              When set to 'disabled', noise suppression is turned off.

          noise_suppression_details: Configuration options for noise suppression. These settings are stored
              regardless of the noise_suppression value, but only take effect when
              noise_suppression is not 'disabled'. If you disable noise suppression and later
              re-enable it, the previously configured settings will be used.

          onnet_t38_passthrough_enabled: Enable on-net T38 if you prefer the sender and receiver negotiating T38 directly
              if both are on the Telnyx network. If this is disabled, Telnyx will be able to
              use T38 on just one leg of the call depending on each leg's settings.

          tags: Tags associated with the connection.

          transport_protocol: One of UDP, TLS, or TCP. Applies only to connections with IP authentication or
              FQDN authentication.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1 or v2.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/ip_connections",
            body=await async_maybe_transform(
                {
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "android_push_credential_id": android_push_credential_id,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "connection_name": connection_name,
                    "default_on_hold_comfort_noise_enabled": default_on_hold_comfort_noise_enabled,
                    "dtmf_type": dtmf_type,
                    "encode_contact_header_enabled": encode_contact_header_enabled,
                    "encrypted_media": encrypted_media,
                    "inbound": inbound,
                    "ios_push_credential_id": ios_push_credential_id,
                    "noise_suppression": noise_suppression,
                    "noise_suppression_details": noise_suppression_details,
                    "onnet_t38_passthrough_enabled": onnet_t38_passthrough_enabled,
                    "outbound": outbound,
                    "rtcp_settings": rtcp_settings,
                    "tags": tags,
                    "transport_protocol": transport_protocol,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                ip_connection_create_params.IPConnectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPConnectionCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPConnectionRetrieveResponse:
        """
        Retrieves the details of an existing ip connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/ip_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPConnectionRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        active: bool | Omit = omit,
        anchorsite_override: AnchorsiteOverride | Omit = omit,
        android_push_credential_id: Optional[str] | Omit = omit,
        call_cost_in_webhooks: bool | Omit = omit,
        connection_name: str | Omit = omit,
        default_on_hold_comfort_noise_enabled: bool | Omit = omit,
        dtmf_type: DtmfType | Omit = omit,
        encode_contact_header_enabled: bool | Omit = omit,
        encrypted_media: Optional[EncryptedMedia] | Omit = omit,
        inbound: InboundIPParam | Omit = omit,
        ios_push_credential_id: Optional[str] | Omit = omit,
        noise_suppression: Literal["inbound", "outbound", "both", "disabled"] | Omit = omit,
        noise_suppression_details: ip_connection_update_params.NoiseSuppressionDetails | Omit = omit,
        onnet_t38_passthrough_enabled: bool | Omit = omit,
        outbound: OutboundIPParam | Omit = omit,
        rtcp_settings: ConnectionRtcpSettingsParam | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        transport_protocol: Literal["UDP", "TCP", "TLS"] | Omit = omit,
        webhook_api_version: Literal["1", "2"] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPConnectionUpdateResponse:
        """
        Updates settings of an existing IP connection.

        Args:
          active: Defaults to true

          anchorsite_override: `Latency` directs Telnyx to route media through the site with the lowest
              round-trip time to the user's connection. Telnyx calculates this time using ICMP
              ping messages. This can be disabled by specifying a site to handle all media.

          android_push_credential_id: The uuid of the push credential for Android

          call_cost_in_webhooks: Specifies if call cost webhooks should be sent for this connection.

          default_on_hold_comfort_noise_enabled: When enabled, Telnyx will generate comfort noise when you place the call on
              hold. If disabled, you will need to generate comfort noise or on hold music to
              avoid RTP timeout.

          dtmf_type: Sets the type of DTMF digits sent from Telnyx to this Connection. Note that DTMF
              digits sent to Telnyx will be accepted in all formats.

          encode_contact_header_enabled: Encode the SIP contact header sent by Telnyx to avoid issues for NAT or ALG
              scenarios.

          encrypted_media: Enable use of SRTP for encryption. Cannot be set if the transport_portocol is
              TLS.

          ios_push_credential_id: The uuid of the push credential for Ios

          noise_suppression: Controls when noise suppression is applied to calls. When set to 'inbound',
              noise suppression is applied to incoming audio. When set to 'outbound', it's
              applied to outgoing audio. When set to 'both', it's applied in both directions.
              When set to 'disabled', noise suppression is turned off.

          noise_suppression_details: Configuration options for noise suppression. These settings are stored
              regardless of the noise_suppression value, but only take effect when
              noise_suppression is not 'disabled'. If you disable noise suppression and later
              re-enable it, the previously configured settings will be used.

          onnet_t38_passthrough_enabled: Enable on-net T38 if you prefer the sender and receiver negotiating T38 directly
              if both are on the Telnyx network. If this is disabled, Telnyx will be able to
              use T38 on just one leg of the call depending on each leg's settings.

          tags: Tags associated with the connection.

          transport_protocol: One of UDP, TLS, or TCP. Applies only to connections with IP authentication or
              FQDN authentication.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1 or v2.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/ip_connections/{id}",
            body=await async_maybe_transform(
                {
                    "active": active,
                    "anchorsite_override": anchorsite_override,
                    "android_push_credential_id": android_push_credential_id,
                    "call_cost_in_webhooks": call_cost_in_webhooks,
                    "connection_name": connection_name,
                    "default_on_hold_comfort_noise_enabled": default_on_hold_comfort_noise_enabled,
                    "dtmf_type": dtmf_type,
                    "encode_contact_header_enabled": encode_contact_header_enabled,
                    "encrypted_media": encrypted_media,
                    "inbound": inbound,
                    "ios_push_credential_id": ios_push_credential_id,
                    "noise_suppression": noise_suppression,
                    "noise_suppression_details": noise_suppression_details,
                    "onnet_t38_passthrough_enabled": onnet_t38_passthrough_enabled,
                    "outbound": outbound,
                    "rtcp_settings": rtcp_settings,
                    "tags": tags,
                    "transport_protocol": transport_protocol,
                    "webhook_api_version": webhook_api_version,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                ip_connection_update_params.IPConnectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPConnectionUpdateResponse,
        )

    def list(
        self,
        *,
        filter: ip_connection_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["created_at", "connection_name", "active"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[IPConnection, AsyncDefaultFlatPagination[IPConnection]]:
        """
        Returns a list of your IP connections.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[connection_name], filter[fqdn], filter[outbound_voice_profile_id],
              filter[outbound.outbound_voice_profile_id]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in ascending order.
                </li>

                <li>
                  <code>-connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/ip_connections",
            page=AsyncDefaultFlatPagination[IPConnection],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    ip_connection_list_params.IPConnectionListParams,
                ),
            ),
            model=IPConnection,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPConnectionDeleteResponse:
        """
        Deletes an existing IP connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/ip_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPConnectionDeleteResponse,
        )


class IPConnectionsResourceWithRawResponse:
    def __init__(self, ip_connections: IPConnectionsResource) -> None:
        self._ip_connections = ip_connections

        self.create = to_raw_response_wrapper(
            ip_connections.create,
        )
        self.retrieve = to_raw_response_wrapper(
            ip_connections.retrieve,
        )
        self.update = to_raw_response_wrapper(
            ip_connections.update,
        )
        self.list = to_raw_response_wrapper(
            ip_connections.list,
        )
        self.delete = to_raw_response_wrapper(
            ip_connections.delete,
        )


class AsyncIPConnectionsResourceWithRawResponse:
    def __init__(self, ip_connections: AsyncIPConnectionsResource) -> None:
        self._ip_connections = ip_connections

        self.create = async_to_raw_response_wrapper(
            ip_connections.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            ip_connections.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            ip_connections.update,
        )
        self.list = async_to_raw_response_wrapper(
            ip_connections.list,
        )
        self.delete = async_to_raw_response_wrapper(
            ip_connections.delete,
        )


class IPConnectionsResourceWithStreamingResponse:
    def __init__(self, ip_connections: IPConnectionsResource) -> None:
        self._ip_connections = ip_connections

        self.create = to_streamed_response_wrapper(
            ip_connections.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            ip_connections.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            ip_connections.update,
        )
        self.list = to_streamed_response_wrapper(
            ip_connections.list,
        )
        self.delete = to_streamed_response_wrapper(
            ip_connections.delete,
        )


class AsyncIPConnectionsResourceWithStreamingResponse:
    def __init__(self, ip_connections: AsyncIPConnectionsResource) -> None:
        self._ip_connections = ip_connections

        self.create = async_to_streamed_response_wrapper(
            ip_connections.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            ip_connections.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            ip_connections.update,
        )
        self.list = async_to_streamed_response_wrapper(
            ip_connections.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            ip_connections.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .porting_order_end_user_admin import PortingOrderEndUserAdmin
from .porting_order_end_user_location import PortingOrderEndUserLocation

__all__ = ["PortingOrderEndUser"]


class PortingOrderEndUser(BaseModel):
    admin: Optional[PortingOrderEndUserAdmin] = None

    location: Optional[PortingOrderEndUserLocation] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .loopcount_param import LoopcountParam as LoopcountParam
from .action_refer_params import ActionReferParams as ActionReferParams
from .action_speak_params import ActionSpeakParams as ActionSpeakParams
from .action_answer_params import ActionAnswerParams as ActionAnswerParams
from .action_bridge_params import ActionBridgeParams as ActionBridgeParams
from .action_gather_params import ActionGatherParams as ActionGatherParams
from .action_hangup_params import ActionHangupParams as ActionHangupParams
from .action_reject_params import ActionRejectParams as ActionRejectParams
from .action_enqueue_params import ActionEnqueueParams as ActionEnqueueParams
from .action_refer_response import ActionReferResponse as ActionReferResponse
from .action_speak_response import ActionSpeakResponse as ActionSpeakResponse
from .action_answer_response import ActionAnswerResponse as ActionAnswerResponse
from .action_bridge_response import ActionBridgeResponse as ActionBridgeResponse
from .action_gather_response import ActionGatherResponse as ActionGatherResponse
from .action_hangup_response import ActionHangupResponse as ActionHangupResponse
from .action_reject_response import ActionRejectResponse as ActionRejectResponse
from .action_transfer_params import ActionTransferParams as ActionTransferParams
from .action_enqueue_response import ActionEnqueueResponse as ActionEnqueueResponse
from .action_send_dtmf_params import ActionSendDtmfParams as ActionSendDtmfParams
from .action_transfer_response import ActionTransferResponse as ActionTransferResponse
from .aws_voice_settings_param import AwsVoiceSettingsParam as AwsVoiceSettingsParam
from .action_leave_queue_params import ActionLeaveQueueParams as ActionLeaveQueueParams
from .action_send_dtmf_response import ActionSendDtmfResponse as ActionSendDtmfResponse
from .action_stop_gather_params import ActionStopGatherParams as ActionStopGatherParams
from .action_stop_siprec_params import ActionStopSiprecParams as ActionStopSiprecParams
from .action_start_siprec_params import ActionStartSiprecParams as ActionStartSiprecParams
from .action_stop_forking_params import ActionStopForkingParams as ActionStopForkingParams
from .transcription_config_param import TranscriptionConfigParam as TranscriptionConfigParam
from .action_leave_queue_response import ActionLeaveQueueResponse as ActionLeaveQueueResponse
from .action_send_sip_info_params import ActionSendSipInfoParams as ActionSendSipInfoParams
from .action_start_forking_params import ActionStartForkingParams as ActionStartForkingParams
from .action_stop_gather_response import ActionStopGatherResponse as ActionStopGatherResponse
from .action_stop_playback_params import ActionStopPlaybackParams as ActionStopPlaybackParams
from .action_stop_siprec_response import ActionStopSiprecResponse as ActionStopSiprecResponse
from .call_control_command_result import CallControlCommandResult as CallControlCommandResult
from .interruption_settings_param import InterruptionSettingsParam as InterruptionSettingsParam
from .telnyx_voice_settings_param import TelnyxVoiceSettingsParam as TelnyxVoiceSettingsParam
from .action_start_playback_params import ActionStartPlaybackParams as ActionStartPlaybackParams
from .action_start_siprec_response import ActionStartSiprecResponse as ActionStartSiprecResponse
from .action_stop_forking_response import ActionStopForkingResponse as ActionStopForkingResponse
from .action_stop_recording_params import ActionStopRecordingParams as ActionStopRecordingParams
from .action_stop_streaming_params import ActionStopStreamingParams as ActionStopStreamingParams
from .action_gather_using_ai_params import ActionGatherUsingAIParams as ActionGatherUsingAIParams
from .action_pause_recording_params import ActionPauseRecordingParams as ActionPauseRecordingParams
from .action_send_sip_info_response import ActionSendSipInfoResponse as ActionSendSipInfoResponse
from .action_start_forking_response import ActionStartForkingResponse as ActionStartForkingResponse
from .action_start_recording_params import ActionStartRecordingParams as ActionStartRecordingParams
from .action_start_streaming_params import ActionStartStreamingParams as ActionStartStreamingParams
from .action_stop_playback_response import ActionStopPlaybackResponse as ActionStopPlaybackResponse
from .google_transcription_language import GoogleTranscriptionLanguage as GoogleTranscriptionLanguage
from .telnyx_transcription_language import TelnyxTranscriptionLanguage as TelnyxTranscriptionLanguage
from .action_resume_recording_params import ActionResumeRecordingParams as ActionResumeRecordingParams
from .action_start_playback_response import ActionStartPlaybackResponse as ActionStartPlaybackResponse
from .action_stop_recording_response import ActionStopRecordingResponse as ActionStopRecordingResponse
from .action_stop_streaming_response import ActionStopStreamingResponse as ActionStopStreamingResponse
from .action_gather_using_ai_response import ActionGatherUsingAIResponse as ActionGatherUsingAIResponse
from .action_pause_recording_response import ActionPauseRecordingResponse as ActionPauseRecordingResponse
from .action_start_recording_response import ActionStartRecordingResponse as ActionStartRecordingResponse
from .action_start_streaming_response import ActionStartStreamingResponse as ActionStartStreamingResponse
from .action_stop_ai_assistant_params import ActionStopAIAssistantParams as ActionStopAIAssistantParams
from .action_gather_using_audio_params import ActionGatherUsingAudioParams as ActionGatherUsingAudioParams
from .action_gather_using_speak_params import ActionGatherUsingSpeakParams as ActionGatherUsingSpeakParams
from .action_resume_recording_response import ActionResumeRecordingResponse as ActionResumeRecordingResponse
from .action_start_ai_assistant_params import ActionStartAIAssistantParams as ActionStartAIAssistantParams
from .action_stop_transcription_params import ActionStopTranscriptionParams as ActionStopTranscriptionParams
from .eleven_labs_voice_settings_param import ElevenLabsVoiceSettingsParam as ElevenLabsVoiceSettingsParam
from .action_start_transcription_params import ActionStartTranscriptionParams as ActionStartTranscriptionParams
from .action_stop_ai_assistant_response import ActionStopAIAssistantResponse as ActionStopAIAssistantResponse
from .action_update_client_state_params import ActionUpdateClientStateParams as ActionUpdateClientStateParams
from .transcription_start_request_param import TranscriptionStartRequestParam as TranscriptionStartRequestParam
from .action_gather_using_audio_response import ActionGatherUsingAudioResponse as ActionGatherUsingAudioResponse
from .action_gather_using_speak_response import ActionGatherUsingSpeakResponse as ActionGatherUsingSpeakResponse
from .action_start_ai_assistant_response import ActionStartAIAssistantResponse as ActionStartAIAssistantResponse
from .action_stop_transcription_response import ActionStopTranscriptionResponse as ActionStopTranscriptionResponse
from .action_start_transcription_response import ActionStartTranscriptionResponse as ActionStartTranscriptionResponse
from .action_update_client_state_response import ActionUpdateClientStateResponse as ActionUpdateClientStateResponse
from .transcription_engine_a_config_param import TranscriptionEngineAConfigParam as TranscriptionEngineAConfigParam
from .transcription_engine_b_config_param import TranscriptionEngineBConfigParam as TranscriptionEngineBConfigParam
from .action_stop_noise_suppression_params import ActionStopNoiseSuppressionParams as ActionStopNoiseSuppressionParams
from .action_switch_supervisor_role_params import ActionSwitchSupervisorRoleParams as ActionSwitchSupervisorRoleParams
from .action_start_noise_suppression_params import (
    ActionStartNoiseSuppressionParams as ActionStartNoiseSuppressionParams,
)
from .action_stop_noise_suppression_response import (
    ActionStopNoiseSuppressionResponse as ActionStopNoiseSuppressionResponse,
)
from .action_switch_supervisor_role_response import (
    ActionSwitchSupervisorRoleResponse as ActionSwitchSupervisorRoleResponse,
)
from .action_start_noise_suppression_response import (
    ActionStartNoiseSuppressionResponse as ActionStartNoiseSuppressionResponse,
)
from .transcription_engine_azure_config_param import (
    TranscriptionEngineAzureConfigParam as TranscriptionEngineAzureConfigParam,
)
from .transcription_engine_google_config_param import (
    TranscriptionEngineGoogleConfigParam as TranscriptionEngineGoogleConfigParam,
)
from .transcription_engine_telnyx_config_param import (
    TranscriptionEngineTelnyxConfigParam as TranscriptionEngineTelnyxConfigParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .comments import (
    CommentsResource,
    AsyncCommentsResource,
    CommentsResourceWithRawResponse,
    AsyncCommentsResourceWithRawResponse,
    CommentsResourceWithStreamingResponse,
    AsyncCommentsResourceWithStreamingResponse,
)
from .porting_orders import (
    PortingOrdersResource,
    AsyncPortingOrdersResource,
    PortingOrdersResourceWithRawResponse,
    AsyncPortingOrdersResourceWithRawResponse,
    PortingOrdersResourceWithStreamingResponse,
    AsyncPortingOrdersResourceWithStreamingResponse,
)
from .activation_jobs import (
    ActivationJobsResource,
    AsyncActivationJobsResource,
    ActivationJobsResourceWithRawResponse,
    AsyncActivationJobsResourceWithRawResponse,
    ActivationJobsResourceWithStreamingResponse,
    AsyncActivationJobsResourceWithStreamingResponse,
)
from .verification_codes import (
    VerificationCodesResource,
    AsyncVerificationCodesResource,
    VerificationCodesResourceWithRawResponse,
    AsyncVerificationCodesResourceWithRawResponse,
    VerificationCodesResourceWithStreamingResponse,
    AsyncVerificationCodesResourceWithStreamingResponse,
)
from .action_requirements import (
    ActionRequirementsResource,
    AsyncActionRequirementsResource,
    ActionRequirementsResourceWithRawResponse,
    AsyncActionRequirementsResourceWithRawResponse,
    ActionRequirementsResourceWithStreamingResponse,
    AsyncActionRequirementsResourceWithStreamingResponse,
)
from .phone_number_blocks import (
    PhoneNumberBlocksResource,
    AsyncPhoneNumberBlocksResource,
    PhoneNumberBlocksResourceWithRawResponse,
    AsyncPhoneNumberBlocksResourceWithRawResponse,
    PhoneNumberBlocksResourceWithStreamingResponse,
    AsyncPhoneNumberBlocksResourceWithStreamingResponse,
)
from .additional_documents import (
    AdditionalDocumentsResource,
    AsyncAdditionalDocumentsResource,
    AdditionalDocumentsResourceWithRawResponse,
    AsyncAdditionalDocumentsResourceWithRawResponse,
    AdditionalDocumentsResourceWithStreamingResponse,
    AsyncAdditionalDocumentsResourceWithStreamingResponse,
)
from .phone_number_extensions import (
    PhoneNumberExtensionsResource,
    AsyncPhoneNumberExtensionsResource,
    PhoneNumberExtensionsResourceWithRawResponse,
    AsyncPhoneNumberExtensionsResourceWithRawResponse,
    PhoneNumberExtensionsResourceWithStreamingResponse,
    AsyncPhoneNumberExtensionsResourceWithStreamingResponse,
)
from .associated_phone_numbers import (
    AssociatedPhoneNumbersResource,
    AsyncAssociatedPhoneNumbersResource,
    AssociatedPhoneNumbersResourceWithRawResponse,
    AsyncAssociatedPhoneNumbersResourceWithRawResponse,
    AssociatedPhoneNumbersResourceWithStreamingResponse,
    AsyncAssociatedPhoneNumbersResourceWithStreamingResponse,
)
from .phone_number_configurations import (
    PhoneNumberConfigurationsResource,
    AsyncPhoneNumberConfigurationsResource,
    PhoneNumberConfigurationsResourceWithRawResponse,
    AsyncPhoneNumberConfigurationsResourceWithRawResponse,
    PhoneNumberConfigurationsResourceWithStreamingResponse,
    AsyncPhoneNumberConfigurationsResourceWithStreamingResponse,
)

__all__ = [
    "PhoneNumberConfigurationsResource",
    "AsyncPhoneNumberConfigurationsResource",
    "PhoneNumberConfigurationsResourceWithRawResponse",
    "AsyncPhoneNumberConfigurationsResourceWithRawResponse",
    "PhoneNumberConfigurationsResourceWithStreamingResponse",
    "AsyncPhoneNumberConfigurationsResourceWithStreamingResponse",
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "ActivationJobsResource",
    "AsyncActivationJobsResource",
    "ActivationJobsResourceWithRawResponse",
    "AsyncActivationJobsResourceWithRawResponse",
    "ActivationJobsResourceWithStreamingResponse",
    "AsyncActivationJobsResourceWithStreamingResponse",
    "AdditionalDocumentsResource",
    "AsyncAdditionalDocumentsResource",
    "AdditionalDocumentsResourceWithRawResponse",
    "AsyncAdditionalDocumentsResourceWithRawResponse",
    "AdditionalDocumentsResourceWithStreamingResponse",
    "AsyncAdditionalDocumentsResourceWithStreamingResponse",
    "CommentsResource",
    "AsyncCommentsResource",
    "CommentsResourceWithRawResponse",
    "AsyncCommentsResourceWithRawResponse",
    "CommentsResourceWithStreamingResponse",
    "AsyncCommentsResourceWithStreamingResponse",
    "VerificationCodesResource",
    "AsyncVerificationCodesResource",
    "VerificationCodesResourceWithRawResponse",
    "AsyncVerificationCodesResourceWithRawResponse",
    "VerificationCodesResourceWithStreamingResponse",
    "AsyncVerificationCodesResourceWithStreamingResponse",
    "ActionRequirementsResource",
    "AsyncActionRequirementsResource",
    "ActionRequirementsResourceWithRawResponse",
    "AsyncActionRequirementsResourceWithRawResponse",
    "ActionRequirementsResourceWithStreamingResponse",
    "AsyncActionRequirementsResourceWithStreamingResponse",
    "AssociatedPhoneNumbersResource",
    "AsyncAssociatedPhoneNumbersResource",
    "AssociatedPhoneNumbersResourceWithRawResponse",
    "AsyncAssociatedPhoneNumbersResourceWithRawResponse",
    "AssociatedPhoneNumbersResourceWithStreamingResponse",
    "AsyncAssociatedPhoneNumbersResourceWithStreamingResponse",
    "PhoneNumberBlocksResource",
    "AsyncPhoneNumberBlocksResource",
    "PhoneNumberBlocksResourceWithRawResponse",
    "AsyncPhoneNumberBlocksResourceWithRawResponse",
    "PhoneNumberBlocksResourceWithStreamingResponse",
    "AsyncPhoneNumberBlocksResourceWithStreamingResponse",
    "PhoneNumberExtensionsResource",
    "AsyncPhoneNumberExtensionsResource",
    "PhoneNumberExtensionsResourceWithRawResponse",
    "AsyncPhoneNumberExtensionsResourceWithRawResponse",
    "PhoneNumberExtensionsResourceWithStreamingResponse",
    "AsyncPhoneNumberExtensionsResourceWithStreamingResponse",
    "PortingOrdersResource",
    "AsyncPortingOrdersResource",
    "PortingOrdersResourceWithRawResponse",
    "AsyncPortingOrdersResourceWithRawResponse",
    "PortingOrdersResourceWithStreamingResponse",
    "AsyncPortingOrdersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["PortingOrderEndUserAdminParam"]


class PortingOrderEndUserAdminParam(TypedDict, total=False):
    account_number: Optional[str]
    """The authorized person's account number with the current service provider"""

    auth_person_name: Optional[str]
    """Name of person authorizing the porting order"""

    billing_phone_number: Optional[str]
    """Billing phone number associated with these phone numbers"""

    business_identifier: Optional[str]
    """European business identification number. Applicable only in the European Union"""

    entity_name: Optional[str]
    """Person Name or Company name requesting the port"""

    pin_passcode: Optional[str]
    """
    PIN/passcode possibly required by the old service provider for extra
    verification
    """

    tax_identifier: Optional[str]
    """European tax identification number. Applicable only in the European Union"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["FaxApplicationListParams", "Filter", "FilterApplicationName"]


class FaxApplicationListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[application_name][contains],
    filter[outbound_voice_profile_id]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]

    sort: Literal["created_at", "application_name", "active"]
    """Specifies the sort order for results.

    By default sorting direction is ascending. To have the results sorted in
    descending order add the <code> -</code> prefix.<br/><br/> That is: <ul>

      <li>
        <code>application_name</code>: sorts the result by the
        <code>application_name</code> field in ascending order.
      </li>

      <li>
        <code>-application_name</code>: sorts the result by the
        <code>application_name</code> field in descending order.
      </li>
    </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.
    """


class FilterApplicationName(TypedDict, total=False):
    """Application name filtering operations"""

    contains: str
    """
    If present, applications with <code>application_name</code> containing the given
    value will be returned. Matching is not case-sensitive. Requires at least three
    characters.
    """


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[application_name][contains], filter[outbound_voice_profile_id]
    """

    application_name: FilterApplicationName
    """Application name filtering operations"""

    outbound_voice_profile_id: str
    """Identifies the associated outbound voice profile."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ActionAcceptSuggestionsParams"]


class ActionAcceptSuggestionsParams(TypedDict, total=False):
    id: str
    """The ID of the address."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .participant_update_params import ParticipantUpdateParams as ParticipantUpdateParams
from .participant_update_response import ParticipantUpdateResponse as ParticipantUpdateResponse
from .participant_retrieve_response import ParticipantRetrieveResponse as ParticipantRetrieveResponse
from .participant_participants_params import ParticipantParticipantsParams as ParticipantParticipantsParams
from .participant_participants_response import ParticipantParticipantsResponse as ParticipantParticipantsResponse
from .participant_retrieve_participants_response import (
    ParticipantRetrieveParticipantsResponse as ParticipantRetrieveParticipantsResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel

__all__ = ["ConnectionListResponse", "Data"]


class Data(BaseModel):
    id: str

    allowed_tools: List[str]

    integration_id: str


class ConnectionListResponse(BaseModel):
    data: List[Data]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .networks import (
    NetworksResource,
    AsyncNetworksResource,
    NetworksResourceWithRawResponse,
    AsyncNetworksResourceWithRawResponse,
    NetworksResourceWithStreamingResponse,
    AsyncNetworksResourceWithStreamingResponse,
)
from .default_gateway import (
    DefaultGatewayResource,
    AsyncDefaultGatewayResource,
    DefaultGatewayResourceWithRawResponse,
    AsyncDefaultGatewayResourceWithRawResponse,
    DefaultGatewayResourceWithStreamingResponse,
    AsyncDefaultGatewayResourceWithStreamingResponse,
)

__all__ = [
    "DefaultGatewayResource",
    "AsyncDefaultGatewayResource",
    "DefaultGatewayResourceWithRawResponse",
    "AsyncDefaultGatewayResourceWithRawResponse",
    "DefaultGatewayResourceWithStreamingResponse",
    "AsyncDefaultGatewayResourceWithStreamingResponse",
    "NetworksResource",
    "AsyncNetworksResource",
    "NetworksResourceWithRawResponse",
    "AsyncNetworksResourceWithRawResponse",
    "NetworksResourceWithStreamingResponse",
    "AsyncNetworksResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = [
    "WebhookDeliveryListResponse",
    "Attempt",
    "AttemptHTTP",
    "AttemptHTTPRequest",
    "AttemptHTTPResponse",
    "Webhook",
]


class AttemptHTTPRequest(BaseModel):
    """Request details."""

    headers: Optional[List[List[str]]] = None
    """List of headers, limited to 10kB."""

    url: Optional[str] = None


class AttemptHTTPResponse(BaseModel):
    """Response details, optional."""

    body: Optional[str] = None
    """Raw response body, limited to 10kB."""

    headers: Optional[List[List[str]]] = None
    """List of headers, limited to 10kB."""

    status: Optional[int] = None


class AttemptHTTP(BaseModel):
    """HTTP request and response information."""

    request: Optional[AttemptHTTPRequest] = None
    """Request details."""

    response: Optional[AttemptHTTPResponse] = None
    """Response details, optional."""


class Attempt(BaseModel):
    """Webhook delivery attempt details."""

    errors: Optional[List[int]] = None
    """Webhook delivery error codes."""

    finished_at: Optional[datetime] = None
    """ISO 8601 timestamp indicating when the attempt has finished."""

    http: Optional[AttemptHTTP] = None
    """HTTP request and response information."""

    started_at: Optional[datetime] = None
    """ISO 8601 timestamp indicating when the attempt was initiated."""

    status: Optional[Literal["delivered", "failed"]] = None


class Webhook(BaseModel):
    """Original webhook JSON data. Payload fields vary according to event type."""

    id: Optional[str] = None
    """Identifies the type of resource."""

    event_type: Optional[str] = None
    """The type of event being delivered."""

    occurred_at: Optional[datetime] = None
    """ISO 8601 datetime of when the event occurred."""

    payload: Optional[Dict[str, object]] = None

    record_type: Optional[Literal["event"]] = None
    """Identifies the type of the resource."""


class WebhookDeliveryListResponse(BaseModel):
    """Record of all attempts to deliver a webhook."""

    id: Optional[str] = None
    """Uniquely identifies the webhook_delivery record."""

    attempts: Optional[List[Attempt]] = None
    """Detailed delivery attempts, ordered by most recent."""

    finished_at: Optional[datetime] = None
    """ISO 8601 timestamp indicating when the last webhook response has been received."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    started_at: Optional[datetime] = None
    """ISO 8601 timestamp indicating when the first request attempt was initiated."""

    status: Optional[Literal["delivered", "failed"]] = None
    """
    Delivery status: 'delivered' when successfuly delivered or 'failed' if all
    attempts have failed.
    """

    user_id: Optional[str] = None
    """Uniquely identifies the user that owns the webhook_delivery record."""

    webhook: Optional[Webhook] = None
    """Original webhook JSON data. Payload fields vary according to event type."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["ConferenceCommandResult"]


class ConferenceCommandResult(BaseModel):
    result: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel
from .integration_secret import IntegrationSecret

__all__ = ["IntegrationSecretCreateResponse"]


class IntegrationSecretCreateResponse(BaseModel):
    data: IntegrationSecret



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["CivicAddressRetrieveResponse", "Data", "DataLocation"]


class DataLocation(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the resource."""

    additional_info: Optional[str] = None

    description: Optional[str] = None

    is_default: Optional[bool] = None
    """Represents whether the location is the default or not."""


class Data(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the resource."""

    city_or_town: Optional[str] = None

    city_or_town_alias: Optional[str] = None

    company_name: Optional[str] = None

    country: Optional[str] = None

    country_or_district: Optional[str] = None

    default_location_id: Optional[str] = None
    """Identifies what is the default location in the list of locations."""

    description: Optional[str] = None

    house_number: Optional[str] = None

    house_number_suffix: Optional[str] = None

    locations: Optional[List[DataLocation]] = None

    postal_or_zip_code: Optional[str] = None

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    state_or_province: Optional[str] = None

    street_name: Optional[str] = None

    street_suffix: Optional[str] = None


class CivicAddressRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["OAuthRetrieveAuthorizeParams"]


class OAuthRetrieveAuthorizeParams(TypedDict, total=False):
    client_id: Required[str]
    """OAuth client identifier"""

    redirect_uri: Required[str]
    """Redirect URI"""

    response_type: Required[Literal["code"]]
    """OAuth response type"""

    code_challenge: str
    """PKCE code challenge"""

    code_challenge_method: Literal["plain", "S256"]
    """PKCE code challenge method"""

    scope: str
    """Space-separated list of requested scopes"""

    state: str
    """State parameter for CSRF protection"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["MigrationParams"]


class MigrationParams(BaseModel):
    source_id: str
    """ID of the Migration Source from which to migrate data."""

    target_bucket_name: str
    """Bucket name to migrate the data into.

    Will default to the same name as the `source_bucket_name`.
    """

    target_region: str
    """Telnyx Cloud Storage region to migrate the data to."""

    id: Optional[str] = None
    """Unique identifier for the data migration."""

    bytes_migrated: Optional[int] = None
    """Total amount of data that has been succesfully migrated."""

    bytes_to_migrate: Optional[int] = None
    """Total amount of data found in source bucket to migrate."""

    created_at: Optional[datetime] = None
    """Time when data migration was created"""

    eta: Optional[datetime] = None
    """Estimated time the migration will complete."""

    last_copy: Optional[datetime] = None
    """Time when data migration was last copied from the source."""

    refresh: Optional[bool] = None
    """
    If true, will continue to poll the source bucket to ensure new data is
    continually migrated over.
    """

    speed: Optional[int] = None
    """Current speed of the migration."""

    status: Optional[Literal["pending", "checking", "migrating", "complete", "error", "stopped"]] = None
    """Status of the migration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .fqdn import Fqdn
from .._models import BaseModel

__all__ = ["FqdnCreateResponse"]


class FqdnCreateResponse(BaseModel):
    data: Optional[Fqdn] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ConferenceRetrieveParams"]


class ConferenceRetrieveParams(TypedDict, total=False):
    region: Literal["Australia", "Europe", "Middle East", "US"]
    """Region where the conference data is located"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["ManagedAccountUpdateGlobalChannelLimitResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """The user ID of the managed account."""

    channel_limit: Optional[int] = None
    """
    Integer value that indicates the number of allocatable global outbound channels
    that are allocated to the managed account. If the value is 0 then the account
    will have no usable channels and will not be able to perform outbound calling.
    """

    email: Optional[str] = None
    """The email of the managed account."""

    manager_account_id: Optional[str] = None
    """The user ID of the manager of the account."""

    record_type: Optional[str] = None
    """The name of the type of data in the response."""


class ManagedAccountUpdateGlobalChannelLimitResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["BucketCreatePresignedURLResponse", "Content"]


class Content(BaseModel):
    token: Optional[str] = None
    """The token for the object"""

    expires_at: Optional[datetime] = None
    """The expiration time of the token"""

    presigned_url: Optional[str] = None
    """The presigned URL for the object"""


class BucketCreatePresignedURLResponse(BaseModel):
    content: Optional[Content] = None
